//      LAPACK++ (V. 1.1)
//      (C) 1992-1996 All Rights Reserved.

// NOTE: This header file includes all of the header files for LAPACK++.
//       IT'S LARGE!

#ifndef _LAPACKPP_H
#define _LAPACKPP_H

#include "lapack.h"

#include "lafnames.h"
#include LA_GEN_MAT_DOUBLE_H
#include LA_VECTOR_DOUBLE_H 
#include LA_GEN_FACT_DOUBLE_H
#include LA_BAND_MAT_DOUBLE_H
#include LA_BAND_FACT_DOUBLE_H
#include LA_TRIDIAG_MAT_DOUBLE_H
#include LA_TRIDIAG_FACT_DOUBLE_H
#include LA_SPD_MAT_DOUBLE_H
#include LA_SYMM_MAT_DOUBLE_H
#include LA_SYMM_FACT_DOUBLE_H
#include LA_SYMM_TRIDIAG_MAT_DOUBLE_H
#include LA_SYMM_BAND_MAT_DOUBLE_H
#include LA_UNIT_UPPER_TRIANG_MAT_DOUBLE_H
#include LA_UNIT_LOWER_TRIANG_MAT_DOUBLE_H
#include LA_SOLVE_DOUBLE_H

#include "arch.h"
#include "laexcp.h"
#include "lautil.h"
#include "blas++.h"
#endif 


